/*

  WizChart
  WizTemp ASCII graphing package
  Copyright (C) 1990,1991,1992 by Network Wizards

*/

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include "wiztemp.h"

#define YBASE    6
#define SIZE_X   80
#define SIZE_Y   63
char chart[SIZE_Y][SIZE_X];

char wiz_dir[SSIZE];
char date[SSIZE];
struct wiz_file file;
int tlo, thi;
int disp_temp;
float tinc, gt_inc, gt_low, gt_hi;
int screen;
float parse_temp();

main(argc,argv)
  int argc;
  char *argv[];
{
  unsigned char c;
  int i;
  float sum, dc, df, dp;
  long sc, cy;
  float hi,lo,fx;

  screen = FALSE;
  disp_temp = DISP_TEMP;
  gt_low = (disp_temp == DEGF) ?  GT_LOWF : GT_LOWC;
  gt_inc = (disp_temp == DEGF) ?  GT_INCF : GT_INCC;
  gt_hi = gt_low + (gt_inc * 50);
  strcpy(wiz_dir,WIZ_DATA_DIR);
  parse_args(argc,argv);
  if (screen)
  {
    gt_hi = gt_low + ((gt_hi - gt_low) * 2.5);
    gt_inc = (gt_hi - gt_low) / 50.0;
  }

  get_data_file(date);
  tlo = atoi(file.lo);
  thi = atoi(file.hi);
  tinc = (thi - tlo) / 256.0;

  clear_chart();
  make_chart();

  hi = -9999.0; lo = 9999.0; fx = 3.95;
  sc = 0; sum = 0;
  for (i = 0; i < WIZ_DATA_SIZE; i++)
  {
    fx = fx + 0.05;
    c = file.data[i];
    if ((c == 0) || (c == 255)) continue;
    dc = tlo + (tinc * c);
    df = (9.0 * dc) / 5.0 + 32;
    dp = (disp_temp == DEGF) ? df : dc;
    sum = sum + dp;
    sc++;
    cy = YBASE + ((dp - gt_low) / gt_inc);
    if ((cy > 0) && (cy < SIZE_Y))
      chart[cy][(int) fx] = '*';

    if (dp > hi) hi = dp;
    if (dp < lo) lo = dp;
  }
  if (sc == 0)
    fatal("No valid temperature data in file");

  cy = YBASE + ((lo - gt_low) / gt_inc);
  if ((cy > 0) && (cy < SIZE_Y))
  {
    chart[cy][77] = 'L';
    chart[cy][78] = 'o';
  }
  cy = YBASE + ((hi - gt_low) / gt_inc);
  if ((cy > 0) && (cy < SIZE_Y))
  {
    chart[cy][77] = 'H';
    chart[cy][78] = 'i';
  }
  cy = YBASE + (((sum/sc) - gt_low) / gt_inc);
  if ((cy > 0) && (cy < SIZE_Y))
  {
    chart[cy][77] = 'A';
    chart[cy][78] = 'v';
  }
  print_chart();
  exit(0);
}


get_data_file(date)
  char *date;
{
  char ftempname[SSIZE];
  int fp;
 
  sprintf(ftempname,"%s%s%s",wiz_dir,DIR_DELIM,date);
#if PC
  _fmode = O_BINARY;
#endif
  if ((fp = open(ftempname,O_RDONLY)) == -1)
  {
    fprintf(stderr,"?Can't find temperature file %s.\n",ftempname);
    die();
  }
#if PC
  _fmode = O_TEXT;
#endif
  if (read(fp,(char *) &file,WIZ_FILE_SIZE) != WIZ_FILE_SIZE)
  {
    fprintf(stderr,"?Temperature file %s has invalid format.\n",ftempname);
    die();
  }
  close(fp);
  return(TRUE);
}   

clear_chart()
{
  int x,y;
  
  for (y = 0; y < SIZE_Y; y++)
    for (x = 0; x < SIZE_X-1; x++)
      chart[y][x] = ' ';
}

make_chart()
{
  int i,j;
  float n;
  char month[3];
  char num[10];

  for (i = 0; i < 2; i++)
    month[i] = date[i+4];
  month[2] = '\0';
  sprintf((chart[60]+34),"%s %.2s, %.4s",montab[atoi(month)],date+6,date);
  sprintf((chart[59]+((80-14-strlen(file.description))/2)),
          "Temperature (%s)",file.description);
  strcpy((chart[3]+35),"Time of Day");
  chart[35][0] = 'T';
  chart[34][0] = 'e';
  chart[33][0] = 'm';
  chart[32][0] = 'p';
  chart[30][0] = (disp_temp == DEGF) ? 'F' : 'C';

  for (i = 4; i < 77; i = i + 12)
    vline(i,6,56);
  for (i = 6; i < 57; i = i + 10)
    hline(i,4,76);

  if (gt_inc > 0.5)
    j = gt_inc * 2;
  else
    j = 1;
  for (n = gt_low; n <= gt_hi; n = n + j)
  {
    i = YBASE + ((n - gt_low) / gt_inc);
    sprintf(num,"%3.0f",n);
    chart[i][1] = num[0];
    chart[i][2] = num[1];
    chart[i][3] = num[2];
  }
  sprintf(chart[5],"    0     2     4     6     8     10    12    14    16    18    20    22    24");
}

print_chart()
{
  int y;
  int my;

  my = (!screen) ? 0 : 4;
  
  for (y = SIZE_Y-1; y > my; y--)
    if (!(screen) ||
         ((screen) && ((y > 58) || (y < 27))))
      printf("%s\n",chart[y]);
}

hline(y,bx,ex)
  int y,bx,ex;
{
  int i,c;

  c = 12;
  for (i = bx; i < (ex + 1); i++)
  {
    if (c++ == 12) 
    {
      chart[y][i] = '+';
      c = 1;
    }
    else 
      chart[y][i] = '-';
  }
}

vline(x,by,ey)
  int x,by,ey;
{
  int i;

  chart[by][x] = '+';
  chart[ey][x] = '+';
  for (i = by + 1; i < ey; i++)
    chart[i][x] = '|';
}


parse_args(argc,argv)
  int argc;
  char *argv[];
{
  int i;
  long now;
  struct tm *timbrk;

  if (argc < 1)
    usage();

  now = time(NULL);
  timbrk = localtime(&now);
  sprintf(date,"%04d%02d%02d",
         (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);

  for (i = 1; i < argc; i++)
  {
    if (*argv[i] != '-')
      usage();
    switch (*(argv[i]+1))
    {
      case 't':
        break;
      case 'y':
        now = now - (long) SECS_DAY;
        timbrk = localtime(&now);
        sprintf(date,"%04d%02d%02d",
               (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
        break;
      case 'd':
        if (++i >= argc) usage();
        strcpy(date,argv[i]);
        if (strlen(date) != 8) usage();
        break;
      case 'F':
        disp_temp = DEGF;
        gt_low = GT_LOWF;
	gt_inc = GT_INCF;
        gt_hi = gt_low + (gt_inc * 50);
	break;
      case 'C':
        disp_temp = DEGC;
        gt_low = GT_LOWC;
	gt_inc = GT_INCC;
        gt_hi = gt_low + (gt_inc * 50);
	break;
      case 'D':
        if (++i >= argc) usage();
        strcpy(wiz_dir,argv[i]);
        break;
      case 'S':
        screen = TRUE;
        break;
      case 's':
        if (++i >= argc) usage();
        gt_low = parse_temp(argv[i]);
        if (++i >= argc) usage();
        gt_hi = parse_temp(argv[i]);
        if (gt_low >= gt_hi)
          fatal("Low temp must be lower than high temp.");
        if (gt_low >= gt_hi) usage();
        gt_inc = (gt_hi - gt_low) / 50.0;
	break;
      default:
	usage();
    }
  }
}

usage()
{
  printf("WizChart -- WizTemp ASCII temperature graphing program\n\n");
  printf("-t                report Today's temperature\n");
  printf("-y                report Yesterday's temperature\n");
  printf("-d yyyymmdd       report temperature of specified date\n");
  printf("-D directory      find temperature file in specified Directory\n");
  printf("-C                report temperature in Celsius\n");
  printf("-F                report temperature in Fahrenheit\n");
  printf("-S                make a Screen sized-chart (24 lines)\n");
  printf("-s low high       scale the chart from low to high degrees\n");
  exit(0);
}

float parse_temp(arg)
  char *arg;
{
  float temp;

  if (sscanf(arg,"%f",&temp) != 1)
    fatal("Temperature must be specified for argument.\n");
  if ((temp < -40.0) || (temp > 200.0))
    fatal("Temps must be between -40 and 200.\n");
  return(temp);
}
