/*

  WizGraph
  WizTemp PostScript graph package
  Copyright (C) 1990,1991,1992 by Network Wizards

*/

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include "wiztemp.h"

char wiz_dir[SSIZE];
char date[SSIZE];
struct wiz_file file;
int tlo, thi;
int disp_temp, avg_plot;
int days_to_plot;
int weekp;
long plot_time;
float tinc, gt_inc, gt_low, gt_hi;
float parse_temp();
struct tm *timbrk;

main(argc,argv)
  int argc;
  char *argv[];
{
  FILE *fp;
  char linbuf[SSIZE];
  char graphfname[SSIZE];
  char month[3];
  unsigned char c;
  int i, n, start, avgc, ac;
  float sum, dc, df, dp, hi, lo, av, step;

  days_to_plot = 1;
  weekp = FALSE;
  avg_plot = AVG_PLOT;
  disp_temp = DISP_TEMP;
  gt_low = (disp_temp == DEGF) ?  GT_LOWF : GT_LOWC;
  gt_inc = (disp_temp == DEGF) ?  GT_INCF : GT_INCC;
  gt_hi = gt_low + (gt_inc * 50);
  strcpy(wiz_dir,WIZ_DATA_DIR);
  parse_args(argc,argv);
  gt_inc = gt_inc * 2;
  if (weekp)
  {
    plot_time = plot_time - ((long) SECS_DAY * 7);
    timbrk = localtime(&plot_time);
    sprintf(date,"%04d%02d%02d",
           (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
  }

  get_data_file(date);
  tlo = atoi(file.lo);
  thi = atoi(file.hi);
  tinc = (thi - tlo) / 256.0;

  sprintf(graphfname,"%s%s%s",WIZ_DATA_DIR,DIR_DELIM,GRAPH_PFX);
  if ((fp = fopen(graphfname,"r")) == NULL)
  {
    fprintf(stderr,"?Can't find prefix file '%s'.\n",graphfname);
    die();
  }
  while (fgets(linbuf,SSIZE,fp) != NULL)
  {
    if (!strncmp(linbuf,"%%INSERT_DEFS",13))
    {
      month[0] = date[4];
      month[1] = date[5];
      month[2] = '\0';
      if (weekp)
        printf("/DATE (Week of %s %.2s, %.4s) def\n",
		montab[atoi(month)],date+6,date);
      else if (days_to_plot == 1)
        printf("/DATE (%s %.2s, %.4s) def\n",montab[atoi(month)],date+6,date);
      else
      {
        long last_day;
	char ldate[40], lmonth[3];

        last_day = plot_time + ((long) SECS_DAY * days_to_plot);
        timbrk = localtime(&last_day);
        sprintf(ldate,"%04d%02d%02d",
    	       (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
        lmonth[0] = ldate[4];
        lmonth[1] = ldate[5];
        lmonth[2] = '\0';
        printf("/DATE (%s %.2s, %.4s thru %s %.2s, %.4s) def\n",
	       montab[atoi(month)],date+6,date,
	       montab[atoi(lmonth)],ldate+6,ldate);
      }
      printf("/TITLE (Temperature (%s)) def\n",file.description);
      printf("/GT_LOW %6.2f def\n",gt_low);
      printf("/GT_INC %6.2f def\n",gt_inc);
      if (disp_temp == DEGF) 
        printf("/V_LABEL (Temp F) def\n");
      else
        printf("/V_LABEL (Temp C) def\n");
    }
    else
      fputs(linbuf,stdout);
  }
  fclose(fp);

  step = 18.72 / gt_inc;
  hi = -9999.0; lo = 9999.0; av = 0.0; ac = 0; avgc = 0;

  for (n = 0; n < days_to_plot; n++)
  {
    start = TRUE;
    for (i = 0; i < WIZ_DATA_SIZE; i++)
    {
      c = file.data[i];
      if ((c == 0) || (c == 255))
      {
	if (avgc)
	{
          sum = sum / avgc;
          printf("%4.3f %4.3f LT\n",i*0.45,(sum-gt_low)*step);
          ac++; av = av + sum;
          if (sum > hi) hi = sum;
          if (sum < lo) lo = sum;
          avgc = 0; sum = 0;
	}
        start = TRUE;
        continue;
      }
      dc = tlo + (tinc * c);
      df = (9 * dc) / 5.0 + 32;
      dp = (disp_temp == DEGF) ? df : dc;
      if (start)
      {
        printf("%4.3f %4.3f MT\n",i*0.45,(dp-gt_low)*step);
        start = FALSE;
        sum = dp;
        avgc = 1;
        continue;
      }
      sum = sum + dp;
      if ((++avgc >= avg_plot) || (i == (WIZ_DATA_SIZE - 1)))
      {
        sum = sum / avgc;
        printf("%4.3f %4.3f LT\n",i*0.45,(sum-gt_low)*step);
        ac++; av = av + sum;
        if (sum > hi) hi = sum;
        if (sum < lo) lo = sum;
        avgc = 0; sum = 0;
      }
    }
    if ((days_to_plot - 1) > n)
    {
      plot_time = plot_time + (long) SECS_DAY;
      timbrk = localtime(&plot_time);
      sprintf(date,"%04d%02d%02d",
             (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
      get_data_file(date);
    }
  }

  printf("648 %4.3f MT 5 0 rlineto 2 -10 rmoveto (Lo ) show (%3.1f) show\n",
	  (lo-gt_low)*step,lo);
  printf("648 %4.3f MT 5 0 rlineto 2 2 rmoveto (Hi ) show (%3.1f) show\n",
	  (hi-gt_low)*step,hi);
  printf("648 %4.3f MT 5 0 rlineto 2 -3 rmoveto (Av ) show (%3.1f) show\n",
	  ((av/ac)-gt_low)*step,av/ac);
  printf("stroke\nshowpage\n");
  exit(0);
}


get_data_file(date)
  char *date;
{
  char ftempname[SSIZE];
  int fp;
 
  sprintf(ftempname,"%s%s%s",wiz_dir,DIR_DELIM,date);
#if PC
  _fmode = O_BINARY;
#endif
  if ((fp = open(ftempname,O_RDONLY)) == -1)
  {
    if (days_to_plot > 1)
    {
      fprintf(stderr,"%%Can't find temperature file %s.\n",ftempname);
      return(FALSE);
    }
    else
    {
      fprintf(stderr,"?Can't find temperature file %s.\n",ftempname);
      die();
    }
  }
#if PC
  _fmode = O_TEXT;
#endif
  if (read(fp,(char *) &file,WIZ_FILE_SIZE) != WIZ_FILE_SIZE)
  {
    fprintf(stderr,"?Temperature file %s has invalid format.\n",ftempname);
    die();
  }
  close(fp);
  return(TRUE);
}   

parse_args(argc,argv)
  int argc;
  char *argv[];
{
  int i;

  if (argc < 1)
    usage();

  plot_time = time(NULL);
  timbrk = localtime(&plot_time);
  sprintf(date,"%04d%02d%02d",
         (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);

  for (i = 1; i < argc; i++)
  {
    if (*argv[i] != '-')
      usage();
    switch (*(argv[i]+1))
    {
      case 't':
        break;
      case 'y':
        plot_time = plot_time - (long) SECS_DAY;
        timbrk = localtime(&plot_time);
        sprintf(date,"%04d%02d%02d",
               (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
        break;
      case 'a':
        if (++i >= argc) usage();
        avg_plot = parse_num(argv[i]);
        break;
      case 'd':
        if (++i >= argc) usage();
        strcpy(date,argv[i]);
        if (strlen(date) != 8) usage();
	if (sscanf(date,"%04d%02d%02d",
	       &timbrk->tm_year,&timbrk->tm_mon,&timbrk->tm_mday) != 3)
	  usage();
	timbrk->tm_sec = 0; timbrk->tm_min = 1; timbrk->tm_hour = 0;
	timbrk->tm_year = timbrk->tm_year - 1900;
	timbrk->tm_mon = timbrk->tm_mon - 1;
        plot_time = mkltime(timbrk);
        break;
      case 'F':
        disp_temp = DEGF;
        gt_low = GT_LOWF;
	gt_inc = GT_INCF;
        gt_hi = gt_low + (gt_inc * 50);
	break;
      case 'C':
        disp_temp = DEGC;
        gt_low = GT_LOWC;
	gt_inc = GT_INCC;
        gt_hi = gt_low + (gt_inc * 50);
	break;
      case 'D':
        if (++i >= argc) usage();
        strcpy(wiz_dir,argv[i]);
        break;
      case 'n':
        if (++i >= argc) usage();
        if (*argv[i] == 'w')
        {
	  weekp = TRUE;
	  days_to_plot = 7;
	}
	else
          days_to_plot = parse_num(argv[i]);
        break;
      case 's':
        if (++i >= argc) usage();
        gt_low = parse_temp(argv[i]);
        if (++i >= argc) usage();
        gt_hi = parse_temp(argv[i]);
        if (gt_low >= gt_hi)
          fatal("Low temp must be lower than high temp.");
        if (gt_low >= gt_hi) usage();
        gt_inc = (gt_hi - gt_low) / 50.0;
	break;
      default:
	usage();
    }
  }
}


usage()
{
  fprintf(stderr,"  Usage: wizgraph [-t] [-y] [-d yyyymmdd] [-n nn|w] [-a nn]\n");
  fprintf(stderr,"             [-C] [-F] [-D directory] [-s low hi]\n");
  exit(0);
}

float parse_temp(arg)
  char *arg;
{
  float temp;

  if (sscanf(arg,"%f",&temp) != 1)
    fatal("Temperature must be specified for argument.\n");
  if ((temp < -40.0) || (temp > 200.0))
    fatal("Temps must be between -40 and 200.\n");
  return(temp);
}

parse_num(arg)
  char *arg;
{
  char *p;
  
  p = arg;
  while (*p != '\0')
    if (!(isdigit(*p++)))
      usage();
  return(atoi(arg));
}
