/*

  WizMon
  WizTemp monitoring program
  Copyright (C) 1992 by Network Wizards

*/

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include "wiztemp.h"
#if UNIX
#include <signal.h>
#include <sys/wait.h>
#include <sys/time.h>
#endif
#if PC
#include <dos.h>
#include <sys/stat.h>
#endif

#define DEBUG       FALSE

int fwizdata;                    /* temperature data file */
int output;			 /* TRUE if should output temp to console */
int file_output;		 /* TRUE if should output temp to a file */
char temp_fmt;
int disp_temp, doy;
int rate, rate_int;
char wiz_port[SSIZE], wiz_dir[SSIZE], wiz_location[SSIZE];
int waiting;
void timeout();

struct alarms {
  float lo;
   int lop;
  float hi;
   int hip;
  float vlo;
   int vlop;
  float vhi;
   int vhip;
  float ideg;
  int imin;
   int incp;
  float ddeg;
  int dmin;
   int decp;
  int prob;
} alarm;

float parse_temp();
float parse_deg();
float old_temp();
stopped();


main(argc,argv)
  int argc;
  char *argv[];
{
  int fo, i, err;
  unsigned char c;
  unsigned int dly;
  float dk, dc, df, dp, old_t, ac;
  time_t now;
  struct tm *timbrk;
  long offset;
  char buf[2];
   
#if UNIX
  signal(SIGCHLD, stopped);
#endif
  output = FALSE;
  file_output = FALSE;
  disp_temp = DISP_TEMP;
  strcpy(wiz_dir,WIZ_DATA_DIR);
  strcpy(wiz_port,WIZ_PORT);
  strcpy(wiz_location,WIZ_LOCATION);
  rate = RATE;
  rate_int = RATE_INT;
  init_alarm(&alarm);
  parse_args(argc,argv);
  temp_fmt = (disp_temp == DEGF) ? 'F' : 'C';
  fwizdata = new_data_file();
  init_port(wiz_port);
  if (file_output)
    fo = open_current(CURRENT_TEMP);
     

  for (;;)  /* each minute */
  {
    err = FALSE;
    ac = 0;
    for (i = 0; i < rate; i++)  /* collect samples at 'rate' */
    {
      now = time(NULL);
      dk = get_temp();
#if DEBUG
      printf("%f ",dk); fflush(stdout);
#endif
      if (dk == -1)             /* error if -1 returned */
        err = TRUE;
      ac = ac + dk;
      dly = rate_int - (time(NULL) - now);
      if (dly > 0)
	sleep(dly);
    }
    if (err)
    { 
      err = FALSE;
      fprintf(stderr,"%%Problem with WizTemp device -- check it.\n");
      if (!alarm.prob)
        alarm_set("prb",0.0,&alarm.prob);
      continue;
    }
    if (alarm.prob)
      alarm_clear("prb",0.0,&alarm.prob);

    dk = (ac / (float) rate) + 0.5;
    dc = dk - ABSZ;
    df = (9 * dc) / 5.0 + 32;
    c  = (dk - TBASE) / TRES;
    if (dk > (TBASE+127)) c = 255;

    /* start new data file if day changed */
    now = time(NULL);
    timbrk = localtime(&now);
    if (doy != timbrk->tm_yday)
    {
      close(fwizdata);
      fwizdata = new_data_file();
    }
    /* save temp in data file */
    offset = (timbrk->tm_hour * 60) + timbrk->tm_min;
    if (lseek(fwizdata,WIZ_DATA_OFFSET+offset,0) == -1)
      fatal("lseek in temperature file failed.");
    *buf = (char) c;
    if (write(fwizdata,buf,1) == -1)
      fatal("write to temperature data file failed.");
 
    dp = (disp_temp == DEGF) ? df : dc;
    if (output)
      printf("%.19s: %5.1f%c\n",ctime(&now),dp,temp_fmt);
    if (file_output)
      output_current(fo,dp);


    /* reset alarms if temp left particular alarm range */
    if ((alarm.lop) && (dc > (alarm.lo + HYST)))
      alarm_clear("l",dp,&alarm.lop);
    if ((alarm.vlop) && (dc > (alarm.vlo + HYST)))
      alarm_clear("vl",dp,&alarm.vlop);
    if ((alarm.hip) && (dc < (alarm.hi - HYST)))
      alarm_clear("h",dp,&alarm.hip);
    if ((alarm.vhip) && (dc < (alarm.vhi - HYST)))
      alarm_clear("vh",dp,&alarm.vhip);

    /* check for new alarms */
    if ((!alarm.lop) && (!alarm.vlop) && (dc <= alarm.lo))
      alarm_set("l",dp,&alarm.lop);
    if ((!alarm.vlop) && (dc <= alarm.vlo))
      alarm_set("vl",dp,&alarm.vlop);
    if ((!alarm.hip) && (!alarm.vhip) && (dc >= alarm.hi))
      alarm_set("h",dp,&alarm.hip);
    if ((!alarm.vhip) && (dc >= alarm.vhi))
      alarm_set("vh",dp,&alarm.vhip);

    /* reset temp change flags if needed */
    /* and check for temperature changing too fast */
    if (alarm.decp != -1)
    {
      old_t = old_temp(dc,offset-alarm.dmin);
      if ((alarm.decp) && ((old_t - dc) < alarm.ddeg))
        alarm_clear("d",dp,&alarm.decp);
      if ((!alarm.decp) && ((old_t - dc) >= alarm.ddeg))
        alarm_set("d",dp,&alarm.decp);
    }

    if (alarm.incp != -1)
    {
      old_t = old_temp(dc,offset-alarm.imin);
      if ((alarm.incp) && ((dc - old_t) < alarm.ideg))
        alarm_clear("i",dp,&alarm.incp);
      if ((!alarm.incp) && ((dc - old_t) >= alarm.ideg))
        alarm_set("i",dp,&alarm.incp);
    }
  }
foo:
  close_port();
  close(fo);
  exit(1);
}


/* return temperature in degc from offset minutes ago */
/* return current temp if can't get old temp */
float old_temp(cdc,offset)
  float cdc;
  long offset;
{
  unsigned char c;
  float v, dc;
  char buf[2];
  int fp;
  char fname[SSIZE];
  time_t now;
  struct tm *timbrk;

  if (offset >= 0L)
  {
    if (lseek(fwizdata,WIZ_DATA_OFFSET+offset,0) == -1)
      fatal("lseek to read old temperature failed");
    if (read(fwizdata,buf,1) != 1)
      fatal("read of old temperature data failed");
    c = (unsigned char) *buf;
  }
  else /* go to previous days file */
  {
    now = time(NULL) - (long) SECS_DAY;
    timbrk = localtime(&now);
    sprintf(fname,"%s%s%04d%02d%02d",wiz_dir,DIR_DELIM,
           (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
    if ((fp = open(fname,O_RDONLY)) == -1)
      return(cdc);
    if (lseek(fp,WIZ_DATA_OFFSET+WIZ_DATA_SIZE+offset,0) == -1)
      fatal("lseek to read old temperature failed");
    if (read(fp,buf,1) != 1)
      fatal("read of old temperature data failed");
    close(fp);
    c = (unsigned char) *buf;
  }

  if ((c == 0) || (c == 255))
    return(cdc);
  dc = (c * TRES) + (TBASE - ABSZ);
  return(dc);
}


int open_current(name)
  char *name;
{
  int file;
  char path[SSIZE];

  sprintf(path,"%s%s%s",wiz_dir,DIR_DELIM,name);
  if ((file = open(path,O_RDWR)) == -1)
  {
    if ((file = creat(path,C_FLAGS)) == -1)
    {
      fprintf(stderr,"?Can't create current temp file '%s'\n",path);
      die();
    }
  }
  return(file);
}

output_current(file,temp)
  int file;
  float temp;
{
  char buf[80];

  if (lseek(file,0L,0) == -1L)
    fatal("Can't seek in current temperature file");
  sprintf(buf,"%5.1f %c\n",temp,temp_fmt);
  if (write(file,buf,strlen(buf)) == -1)
    fatal("Can't write to current temperature file");
}


/*
  Start a new datafile if needed.
  Data is 1440 bytes, one for each minute of the day.
  Each byte is the value returned by the WizTemp, not a temperature.
*/
int new_data_file()
{
  int i, fp;
  time_t now;
  struct tm *timbrk;
  char fwizdataname[SSIZE];
  struct wiz_file file;
 
#if DEBUG
  printf("new_data_file()\n");
#endif
  now = time(NULL);
  timbrk = localtime(&now);
  doy = timbrk->tm_yday;

#if PC
  _fmode = O_BINARY;
#endif
  sprintf(fwizdataname,"%s%s%04d%02d%02d",wiz_dir,DIR_DELIM,
         (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
  if ((fp = open(fwizdataname,O_RDWR)) == -1)
  {
    if ((fp = creat(fwizdataname,C_FLAGS)) == -1)
    {
      fprintf(stderr,"Can't create temperature file '%s'\n.",fwizdataname);
      die();
    }

    sprintf(file.date,"%04d%02d%02d",
           (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
    strcpy(file.description,wiz_location);
    sprintf(file.lo,"%-.0d",(TBASE-ABSZ));
    sprintf(file.hi,"%-.0d",(TBASE-ABSZ) + 128);
    strcpy(file.rate,"60");
    for (i = 0; i < WIZ_FILLER_SIZE; i++)
      file.filler[i] = 0;   
    for (i = 0; i < WIZ_DATA_SIZE; i++)
      file.data[i] = 0;   

    if (write(fp,(char *) &file,WIZ_FILE_SIZE) == -1)
    {
      fprintf(stderr,"Can't write new temperature file '%s'\n.",fwizdataname);
      die();
    }
   close(fp);
   if ((fp = open(fwizdataname,O_RDWR)) == -1)
     fatal("Can't open new temperature data file");
   }
#if PC
   _fmode = O_TEXT;
#endif
   return(fp);
}   


parse_args(argc,argv)
  int argc;
  char *argv[];
{
  int i;

  for (i = 1; i < argc; i++)
  {
    if (*argv[i] != '-')
      usage();
    switch (*(argv[i]+1))
    {
      case 'o':
        output = TRUE;
        break;
      case 'f':
        file_output = TRUE;
        break;
      case 'p':
	if (++i >= argc)
          usage();
        strcpy(wiz_port,argv[i]);
	break;
      case 'D':
	if (++i >= argc)
          usage();
        strcpy(wiz_dir,argv[i]);
        break;
      case 'L':
	if (++i >= argc)
          usage();
        strcpy(wiz_location,argv[i]);
        break;
      case 'F':
        disp_temp = DEGF;
	break;
      case 'C':
        disp_temp = DEGC;
	break;
      case 'r':
	if (++i >= argc)
          usage();
        rate = parse_min(argv[i]);
        if (rate > 30) 
          fatal("Rate must be less than 30");
        rate_int = 60 / rate;
        break;
      case 'l':
	if (++i >= argc)
          usage();
        alarm.lo = parse_temp(argv[i]);
	break;
      case 'h':
	if (++i >= argc)
          usage();
        alarm.hi = parse_temp(argv[i]);
	break;
      case 'v': 
        switch (*(argv[i]+2))
	{
	case 'l':
          if (++i >= argc)
            usage();
          alarm.vlo = parse_temp(argv[i]);
	  break;
	case 'h':
          if (++i >= argc)
            usage();
          alarm.vhi = parse_temp(argv[i]);
	  break;
	default:
	  fprintf(stderr,"?Unknown switch: %s\n",argv[i]);
	  exit(1);
	}
	break;
      case 'i':
        if (++i >= argc)
          usage();
        alarm.ideg = parse_deg(argv[i]);
        if (++i >= argc)
          usage();
	alarm.imin = parse_min(argv[i]);
        alarm.incp = FALSE;
	break;
      case 'd':
        if (++i >= argc)
          usage();
        alarm.ddeg = parse_deg(argv[i]);
        if (++i >= argc)
          usage();
	alarm.dmin = parse_min(argv[i]);
        alarm.decp = FALSE;
	break;
      default:
	usage();
    }
  }
}

usage()
{
  printf("WizMon -- WizTemp-2 temperature monitoring process\n\n");
  printf("-o             Output temp to console (besides to data file)\n");
  printf("-f             output current temperature to a text File too\n");
  printf("-p port/tty    port WizTemp is on (/dev/ttyx, COM1)\n");
  printf("-D directory   put temperature file in specified Directory\n");
  printf("-L location    short description of WizTemp location (32 chars)\n");
  printf("-r N           take N samples a minute for each temp recording\n");
  printf("-C             set temperatures in Celsius\n");
  printf("-F             set temperatures in Fahrenheit\n");
  printf("-l D           set Low temp alarm at D degrees\n");
  printf("-vl D          set Very Low temp alarm at D degrees\n");
  printf("-d D M         set Decreasing alarm to D degrees in M minutes\n");
  printf("-h D           set High temp alarm at D degrees\n");
  printf("-vh D          set Very High temp alarm at D degrees\n");
  printf("-i D M         set Increasing alarm to D degrees in M minutes\n");
  exit(1);
}


init_alarm(alarmp)
  struct alarms *alarmp;
{
  alarmp->lo = -999.0;
   alarmp->lop = FALSE;
  alarmp->hi = 999.0;
   alarmp->hip = FALSE;
  alarmp->vlo = -999.0;
   alarmp->vlop = FALSE;
  alarmp->vhi = 999.0;
   alarmp->vhip = FALSE;
  alarmp->ideg = 999.0;
  alarmp->imin = 1;
   alarmp->incp = -1;
  alarmp->ddeg = 999.0;
  alarmp->dmin = 1;
   alarmp->decp = -1;
  alarmp->prob = FALSE;
}

alarm_set(type,dp,setp)
  char *type;
  float dp;
  int *setp;
{
  char name[32];

  *setp = TRUE;
  printf("\007** \007Alarm\007 '%s' at %3.1f%c\n",type,dp,temp_fmt);  
#if UNIX
  strcpy(name,"alarm.");
  strcat(name,type);
  run(name,dp);
#endif
}

alarm_clear(type,dp,setp)
  char *type;
  float dp;
  int *setp;
{
  char name[32];

  *setp = FALSE;
  printf("\007** Clear alarm '%s' at %3.1f%c\n",type,dp,temp_fmt);  
#if UNIX
  strcpy(name,"unalarm.");
  strcat(name,type);
  run(name,dp);
#endif
}

#if UNIX
run(name,dp)
  char *name;
  float dp;
{
  char path[SSIZE], temps[16], tfmt[8];
  int pid;

  sprintf(path,"%s%s%s",
          WIZ_ALARM_DIR,DIR_DELIM,name);
  sprintf(temps,"%.1f",dp);
  tfmt[0] = temp_fmt;  tfmt[1] = '\0';

  if ((pid = fork()) == -1)
    fatal("Can't start subprocess for alarm");
  if (pid == 0)  /* child */
  {
    if (execl(path,name,temps,tfmt,wiz_location,NULL) == -1)
    fprintf(stderr,"%%Can't execute alarm '%s'\n",path);
    die();
  }
}

stopped()
{
  int pid;
 
  pid = wait(0);
}
#endif


/* return temp in DEGC always */
float parse_temp(arg)
  char *arg;
{
  float temp;

  if (sscanf(arg,"%f",&temp) != 1)
    fatal("Temperature must be specified for argument.\n");
  if ((temp < -40.0) || (temp > 200.0))
    fatal("Temps must be between -40 and 200.\n");
  if (disp_temp == DEGF)
    temp = (temp - 32.0) * 5.0 / 9.0;
  return(temp);
}

float parse_deg(arg)
  char *arg;
{
  float temp;

  if (sscanf(arg,"%f",&temp) != 1)
    fatal("Degrees must be specified for argument.\n");
  if ((temp < 0.1) || (temp > 100.0))
    fatal("Degrees must be between 0.1 and 100.\n");
  return(temp);
}

parse_min(arg)
  char *arg;
{
  char *p;
  
  p = arg;
  while (*p != '\0')
    if (!(isdigit(*p++)))
      usage();
  return(atoi(arg));
}
