/*
  WizTemp header file
  Copyright (C) 1990,1991,1992,1993 by Network Wizards
*/

/* Don't touch these */
#define VERSION     3.2
#define FALSE       0
#define TRUE        1
#define OFF         FALSE
#define ON          TRUE
#define DEGC        0
#define DEGF        1
#define ABSZ        273
#define TBASE       233
#define TRES        0.5

/* System type.  PC not fully implemented. */
#define UNIX        TRUE
#define PC	    FALSE


/*
 WizTemp Model number set here.  Set it to 1 or 2, if you have the
 WizTemp-1 or WizTemp-2, respectively.
*/
#define MODEL       2


/*
 NOTE:  The following HAVE_* defines are set for use of a WizTemp-2
 under SunOS 4.*.  If this is your configuration you should not need
 to change any of them.
*/

/*
 This program can set the communication parameters for the RS-232 line
 in two different ways.  Your unix may have the sgtty interface or the
 termios interface, or maybe both (like SunOS 4.*).  Check man termio,
 termios, and sgtty to find out which your system supports.  Set one of
 them to TRUE, the other FALSE.  If you have neither, things might work
 if you set both to FALSE.  On Suns and SGIs, use TERMIOS.  Under
 Ultrix 4.2, use HAVE_SGTTY.
*/
#define HAVE_SGTTY   FALSE
#define HAVE_TERMIOS TRUE


/* DTR/RTS serial port signal control:

 The WizTemp-1 requires on/off control of the DTR signal on the serial
 port to operate.  The WizTemp-2 gets power from the computers' DTR and RTS
 lines, at least one of which must be set on.  In either of these cases
 this software must be able to control DTR (and maybe RTS).

 Normally, both these lines are turned on when the serial port is
 opened, which means you can set both these defines to FALSE and still
 expect the WizTemp-2 to get power.  However, if you want to be sure
 they are set on, or, if you have a WizTemp-1, you will need to set one
 of these options to TRUE.

 If you have the TIOCSDTR (and TIOCCDTR) ioctls, set HAVE_SDTR to TRUE.
 If you have the TIOCMBIS (and TIOCMBIC) ioctls, set HAVE_MBIS to TRUE.
 Generally, if you said above that you HAVE_SGTTY, then you probably
 HAVE_SDTR.  If you said you HAVE_TERMIOS, then you probably HAVE_MBIS.
 Only set one of these to TRUE.
*/
#define HAVE_SDTR            FALSE
#define HAVE_MBIS            TRUE


/*
 There are at least two standard ways to poll devices in Unix, one uses
 the select call (BSD), the other uses poll (SysV).  Set the one your 
 system supports to TRUE, and the rest to FALSE.  If your system has 
 neither, set them both to FALSE; in this mode programs reading data
 from the WizTemp may hang if no data is returned (hardware failure,
 disconnected serial line, etc).  You can check "man select" and
 "man poll" to see which one your system supports.  SunOS 4.* supports
 the select call.  The poll code is not yet implemented.
*/
#define HAVE_SELECT     TRUE
#define HAVE_POLL       FALSE


/*
 This software supports three different ways of using a function defined
 in ANSI C as mktime().  Under SunOS 4.* this was named timelocal(). 
 Under Solaris 2.*, use HAVE_MKTIME.  Otherwise, check the man pages to
 see which one you have, and set it to TRUE and the others to FALSE.
 If you have neither, set HAVE_NOTIME to TRUE and an internal routine
 will be used.
*/
#define HAVE_TIMELOCAL  TRUE
#define HAVE_MKTIME     FALSE
#define HAVE_NOTIME     FALSE


/*
 Set your default WizTemp directory paths here.  The data directory
 holds all your daily temperature files recorded by the wizmon program.
 The alarm directory contains shell scripts or programs to execute
 when an abnormal condition is detected.
*/
#if UNIX
#define WIZ_DATA_DIR   "/usr/local/wiztemp/data"
#define WIZ_ALARM_DIR  "/usr/local/wiztemp/alarm"
#define DIR_DELIM      "/"
#endif
#if PC
#define WIZ_DIR        "C:\\WIZTEMP\\DATA"
#define WIZ_ALARM_DIR  "C:\\WIZTEMP\\ALARM"
#define DIR_DELIM       "\\"
#endif


/*
 Terminal line the WizTemp is connected to.  It is recommended that
 you make a symbolic link for /dev/wiztemp to /dev/ttyXXX.
*/
#if UNIX
#define WIZ_PORT   "/dev/wiztemp"
#endif
#if PC
#define WIZ_PORT   "COM2"
#endif


/*
 Default location string if not specified.  Also set with -L option.
 This string is displayed by all the temperature reporting programs.
 Set it to be a description of where the sensor is located (31 chars max).
*/
#define WIZ_LOCATION "location not set"


/* Default temperature output format, set to DEGC or DEGF */
/* also set with -C or -F option */
#define DISP_TEMP   DEGF


/*
 Default wizmon sample rate (per minute).  For still air, 1 is ok.
 For moving air (A/C breeze in machine room, or outdoors) use 3 or 4.
 Also set with -r option.
*/
#define RATE        3
#define RATE_INT    (60 / RATE)


/*
 Alarm hysteresis in degrees centigrade.  Alarm will clear when temp goes
 above (or below) its setting plus (or minus) HYST degrees.
*/
#define HYST        2      /* degrees centigrade */


/* WizGraph: default number of minutes to average for each point plotted */
/* also set with -a option */
#define AVG_PLOT    6


/* default WizGraph/Chart output range stuff (low temp, increment) */
/* also set with -s option */
/* Fahrenheit */
#define GT_LOWF     60.0
#define GT_INCF     0.5
/* Celsius */
#define GT_LOWC     15.0
#define GT_INCC     0.3


/* WizMon: name of file current temperature is kept in if using -f option */
#define CURRENT_TEMP "current"


/**********
   Don't play with things past this line unless you know what you're doing.
**********/

/* declarations for wiztemp.c library */
init_port();
float get_temp();
close_port();
time_t mkltime();
fatal();
die();
extern char *montab[];

/* flags for data file creation */
#if UNIX
#define C_FLAGS    0664
#endif
#if PC
#define C_FLAGS    (S_IWRITE | S_IREAD)
#endif

/* WizGraph prepend file name, in WIZ_DATA_DIR directory. */
#define GRAPH_PFX   "graph.ps"

#define SSIZE       256          /* generic strings length */
#define SECS_DAY    (24.0*60.0*60.0)

#define WIZ_FILE_SIZE   2048
#define WIZ_DATA_SIZE   1440
#define WIZ_DATA_OFFSET 512
#define WIZ_FILLER_SIZE (WIZ_DATA_OFFSET-9-32-8-8-8)

/* format of wiztemp data files */
struct wiz_file {
  char date[9];         /* yyyymmdd<null> */
  char description[32]; /* description<null> */
  char lo[8];           /* nnn<null> */
  char hi[8];           /* nnn<null> */
  char rate[8];         /* nnnnnnn<null> sample rate in seconds */
  char filler[WIZ_FILLER_SIZE];       /* for future use */
  unsigned char data[WIZ_DATA_SIZE];  /* starts at offset 512 */
  char trailer[WIZ_FILE_SIZE-WIZ_DATA_OFFSET-WIZ_DATA_SIZE];
};
