/*

  WizWhat
  WizTemp statistics generator
  Copyright (C) 1990,1991,1992 by Network Wizards

*/

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include "wiztemp.h"

#define DEBUG       FALSE

char wiz_dir[SSIZE];
char date[SSIZE];
struct wiz_file file;
int tlo, thi;
float tinc;

main(argc,argv)
  int argc;
  char *argv[];
{
  int i;
  unsigned char c;
  long lo,lo_t, hi,hi_t, la,la_t, av,av_t;
  char mon[3];
   
  strcpy(wiz_dir,WIZ_DATA_DIR);
  parse_args(argc,argv);
  get_data_file(date);

  tlo = atoi(file.lo);
  thi = atoi(file.hi);
  tinc = (thi - tlo) / 256.0;
  lo = 255; hi = 0; av = 0; la = 0; av_t = 0; lo_t = 0; hi_t = 0; la_t = 0;
  for (i = 0; i < WIZ_DATA_SIZE; i++)
  {
    c = file.data[i];
    if ((c != 0) && (c != 255))
    {
      if (c <= lo) {lo = c; lo_t = i;}
      if (c >= hi) {hi = c; hi_t = i;}
      av = av + c; av_t++;
      la = c; la_t = i;
    }
  }
  if (av_t == 0)
  {
    printf("?No valid temperature data in file.\n");
    exit(0);
  }
  av = ((float) av / (float) av_t) + 0.5;
   mon[0] = date[4]; mon[1] = date[5]; mon[2] = '\0';
  printf("Temperature statistics for %s %.2s, %.4s (%s)\r\n",
	  montab[atoi(mon)],date+6,date,file.description);
  printf("last at "); disp_time(la_t); printf(": "); disp_temp(la); crlf();
  printf("high at "); disp_time(hi_t); printf(": "); disp_temp(hi); crlf();
  printf("low  at "); disp_time(lo_t); printf(": "); disp_temp(lo); crlf();
  printf("average temp : "); disp_temp(av); crlf();
  exit(0);
}


disp_time(t)
  long t;
{
  int hours, mins;

  hours = t / 60;
  mins  = t - (hours * 60);
  printf("%02d:%02d",hours,mins);
}

disp_temp(t)
  long t;
{
  float dc, df;

  dc = tlo + (tinc * t);
  df = (9 * dc) / 5.0 + 32;
  printf("%6.1fC   %6.1fF",dc,df);
}

crlf()
{
  putchar('\r');
  putchar('\n');
  fflush(stdout);
}

get_data_file(date)
  char *date;
{
  char ftempname[SSIZE];
  int fp;
 
  sprintf(ftempname,"%s%s%s",wiz_dir,DIR_DELIM,date);
#if PC
  _fmode = O_BINARY;
#endif
  if ((fp = open(ftempname,O_RDONLY)) == -1)
  {
    fprintf(stderr,"?Can't find temperature file %s.\n",ftempname);
    die();
  }
#if PC
  _fmode = O_TEXT;
#endif
  if (read(fp,(char *) &file,WIZ_FILE_SIZE) != WIZ_FILE_SIZE)
  {
    fprintf(stderr,"?Temperature file %s has invalid format.\n",ftempname);
    die();
  }
  close(fp);
  return(TRUE);
}   


parse_args(argc,argv)
  int argc;
  char *argv[];
{
  int i;
  long now;
  struct tm *timbrk;

  if (argc < 1)
    usage();

  now = time(NULL);
  timbrk = localtime(&now);
  sprintf(date,"%04d%02d%02d",
         (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);

  for (i = 1; i < argc; i++)
  {
    if (*argv[i] != '-')
      usage();
    switch (*(argv[i]+1))
    {
      case 't':
        break;
      case 'y':
        now = now - (long) SECS_DAY;
        timbrk = localtime(&now);
        sprintf(date,"%04d%02d%02d",
               (timbrk->tm_year)+1900,(timbrk->tm_mon)+1,timbrk->tm_mday);
        break;
      case 'd':
        if (++i >= argc) usage();
        strcpy(date,argv[i]);
        if (strlen(date) != 8) usage();
        break;
      case 'D':
        if (++i >= argc) usage();
        strcpy(wiz_dir,argv[i]);
        break;
      default:
        usage();
    }
  }
}

usage()
{
  printf("WizWhat -- WizTemp reporting program\n\n");
  printf("-t                report Today's temperature\n");
  printf("-y                report Yesterday's temperature\n");
  printf("-d yyyymmdd       report temperature of specified date\n");
  printf("-D directory      find temperature file in specified Directory\n");
  exit(0);
}
