/*
   beta release -- 10 July 1991

   xtemp -- X windows real-time temperature graph
   Copyright (C) 1991 Network Wizards
*/

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#include "wiztemp.h"
#include "xtemp.ico"

#define FALSE 0
#define TRUE  1

#define SMALL 1
#define OK    0

#define CENTER_F         75
#define CENTER_C         23
#define DELAY            60       /* update interval in seconds */
#define SHIFT_INTERVAL   60	  /* shift graph left this much (mins) */
#define FONT             "5x8"
#define FONT_X           5
#define FONT_Y           8
#define SCALE_YPOS       4
#define LOC_YPOS         0
#define GRAPH_LEF_BORDER (4 + (FONT_X * 2))
#define GRAPH_RIG_BORDER (4 + (FONT_X * 2))
#define GRAPH_TOP_BORDER (LOC_YPOS + 4)
#define GRAPH_BOT_BORDER (SCALE_YPOS + FONT_Y + 4)
#define WIN_SIZE_X       ((4 * 60) + (GRAPH_LEF_BORDER + GRAPH_RIG_BORDER) + 1)
#define WIN_MIN_SIZE_X   (WIN_SIZE_X - 60)
#define WIN_SIZE_Y       (60 + GRAPH_TOP_BORDER + GRAPH_BOT_BORDER)
#define WIN_MIN_SIZE_Y   (WIN_SIZE_Y - 20)
#define MAX_TEMPS        2048

FILE *tempfp;
void xget_temp();

int graphx, graphy, graphsx, graphsy;

int temps[MAX_TEMPS];
int temp_mins, temp_ptr, temp_time;
int temp_center = 75;
int temp_base = 60;
int temp_scale = 2;
int temp_inc = 10;
char forc = 'F';
char s_temp[10];
int l_temp, w_temp, o_temp;
time_t now;
struct tm *tmptr;

Window root, child;

Display *display;
int screen;
Window win;
int winsx, winsy;
int winposx, winposy;
GC whitetextgc, blacktextgc, whitegc, blackgc;
XFontStruct *font_info;
char temp_file[128];
char window_name[128];
char display_name[128] = NULL;

main(argc, argv)
  int argc;
  char *argv[];
{
  unsigned int border_width = 4;
  unsigned int display_width, display_height;
  Pixmap icon_pixmap;
  XSizeHints size_hints;
  XEvent report;
  int window_size = OK;
  int i;

  sprintf(window_name,"Temperature (%s)",WIZ_LOCATION);
  for (i = 1; i < argc; i++)
  {
    if (*argv[i] == '-')
    {
      switch (argv[i][1])
      {
        case 'd':
	  i++;
	  strcpy(display_name,argv[i]);
	  break;
        case 'n':
	  i++;
	  strcpy(window_name,argv[i]);
	  break;
        default:
	  fprintf(stderr,"Unknown option %s\n",argv[i]);
	  exit(1);
	  break;
      }
    }
  }

  sprintf(temp_file,"%s%s%s",WIZ_DATA_DIR,DIR_DELIM,CURRENT_TEMP);
  if ((tempfp = fopen(temp_file,"r")) == NULL)
  {
    fprintf(stderr,"Temperature file %s not found.\n",temp_file);
    exit(1);
  }

  /* connect to X server */
  if ((display = XOpenDisplay(display_name)) == NULL)
  {
    fprintf(stderr,"Can't connect to server %s\n",
	    XDisplayName(display_name));
    exit(1);
  }

  /* get screen size */
  screen = DefaultScreen(display);

  display_width  = DisplayWidth(display, screen);
  display_height = DisplayHeight(display, screen);

  /* compute size of window */
  winsx = WIN_SIZE_X;
  winsy = WIN_SIZE_Y;
  winposx = 0;
  winposy = 0;

  for (i = 0; i < MAX_TEMPS; i++)
    temps[i] = 0;
  time(&now);
  tmptr = localtime(&now);
  temp_mins = tmptr->tm_hour * 60;
  temp_time = now - (tmptr->tm_min * 60);

  configure();

  /* create opaque window */
  win = XCreateSimpleWindow(display, RootWindow(display,screen),
			    winposx, winposy, winsx, winsy, border_width,
			    BlackPixel(display, screen),
			    WhitePixel(display, screen));

  /* create pixmap for icon */
  icon_pixmap = XCreateBitmapFromData(display, win, icon_bits,
                      		      icon_width, icon_height);

  /* initialize size hint property for window manager */
  size_hints.flags = (PPosition | PSize | PResizeInc | PMinSize | PMaxSize);
  size_hints.x = winposx;
  size_hints.y = winposy;
  size_hints.width = winsx;
  size_hints.height = winsy;
  size_hints.min_width  = WIN_MIN_SIZE_X;
  size_hints.min_height = WIN_MIN_SIZE_Y;
  size_hints.width_inc = 10;
  size_hints.height_inc = 10;
  size_hints.max_width = 1280;
  size_hints.max_height = 360;

  /* set properties for window manager */
  XSetStandardProperties(display, win, window_name, icon_name,
			 icon_pixmap, argv, argc, &size_hints);

  /* select event types wanted */
  XSelectInput(display, win, (ExposureMask | StructureNotifyMask));

  /* load a font */
  if ((font_info = XLoadQueryFont(display,FONT)) == NULL)
  {
    fprintf(stderr,"Can't load font %s\n",FONT);
    exit(1);
  }

  /* create GCs for text and drawing */
  get_blacktextGC(&blacktextgc);
  get_whitetextGC(&whitetextgc);
  get_whiteGC(&whitegc);
  get_blackGC(&blackgc);

  /* display window */
  XMapWindow(display, win);

  XSync(display);

  signal(SIGALRM,xget_temp);
  xget_temp();

  for (;;)
  {  
    XNextEvent(display, &report);
    switch (report.type) 
    {
      case Expose:
        /* remove all other expose events on queue */
        while (XCheckTypedEvent(display, Expose, &report));
        if (window_size == SMALL)
	{
	  TooSmall(blacktextgc);
	}
        else
	{
	  redraw();
	}
        break;
      case ConfigureNotify:
        winsx = report.xconfigure.width;
        winsy = report.xconfigure.height;
        if ((winsx < size_hints.min_width) ||
	    (winsy < size_hints.min_height))
	  window_size = SMALL;
        else
	  window_size = OK;
	configure();
        break;
      default:
        break;
    }
  }
foo:
  XUnloadFont(display, font_info->fid);
  XFreeGC(display, blackgc);
  XFreeGC(display, whitegc);
  XFreeGC(display, blacktextgc);
  XFreeGC(display, whitetextgc);
  XCloseDisplay(display);
  fclose(tempfp);
  exit(0);
}

/* here when window size changes */
configure()
{
  int i, offset;

  graphx  = GRAPH_LEF_BORDER;
  graphy  = GRAPH_TOP_BORDER;
  graphsx = winsx - graphx - GRAPH_RIG_BORDER;
  graphsy = winsy - graphy - GRAPH_BOT_BORDER;
  if (forc == 'F')
  {
    temp_center = CENTER_F;
    temp_scale = 2;
    temp_inc = 10;
  }
  else
  if (forc == 'C')
  {
    temp_center = CENTER_C;
    temp_scale = 4;
    temp_inc = 5;
  }
  temp_base = temp_center - ((graphsy / temp_scale) / 2);
  temp_base -= (temp_base % temp_inc);
  if (temp_base < 0)
    temp_base = 0;
/*
printf("graphx %d, graphy %d, graphsx %d, graphsy %d, winsx %d, winsy %d\n",graphx,graphy,graphsx,graphsy,winsx,winsy);
*/

  time(&now);
  temp_ptr = (now - temp_time) / 60;
  if (temp_ptr >= graphsx)
  {
    offset = ((temp_ptr - graphsx) + 60) / 60;
    temp_mins += (offset * SHIFT_INTERVAL);
    if (temp_mins >= (24 * 60))
      temp_mins -= (24 * 60);
    temp_time += (60 * offset * SHIFT_INTERVAL);
    temp_ptr = (now - temp_time) / 60;
    for (i = 0; i <= temp_ptr; i++)
      temps[i] = temps[i+(offset * SHIFT_INTERVAL)];
    for (; i < MAX_TEMPS; i++)
      temps[i] = 0;
    redraw();
  }

}

get_blacktextGC(gc)
  GC *gc;
{
  XGCValues values;

  *gc = XCreateGC(display, win, 0, &values);
  XSetFont(display, *gc, font_info->fid);
  XSetForeground(display, *gc, BlackPixel(display,screen));
}

get_whitetextGC(gc)
  GC *gc;
{
  XGCValues values;

  *gc = XCreateGC(display, win, 0, &values);
  XSetFont(display, *gc, font_info->fid);
  XSetForeground(display, *gc, WhitePixel(display,screen));
}

get_whiteGC(gc)
  GC *gc;
{
  XGCValues values;
  unsigned int line_width = 1;
  int line_style = LineSolid;
  int cap_style = CapButt;
  int join_style = JoinRound;

  *gc = XCreateGC(display, win, 0, &values);
  XSetForeground(display, *gc, WhitePixel(display,screen));
  XSetBackground(display, *gc, BlackPixel(display,screen));
  XSetLineAttributes(display, *gc, line_width, line_style, 
		     cap_style, join_style);
}

get_blackGC(gc)
  GC *gc;
{
  XGCValues values;
  unsigned int line_width = 1;
  int line_style = LineSolid;
  int cap_style = CapButt;
  int join_style = JoinRound;

  *gc = XCreateGC(display, win, 0, &values);
  XSetForeground(display, *gc, BlackPixel(display,screen));
  XSetBackground(display, *gc, WhitePixel(display,screen));
  XSetLineAttributes(display, *gc, line_width, line_style, 
		     cap_style, join_style);
}

void xget_temp()
{
  int i, deg, t;
  float ftemp;

  alarm(DELAY);			/* get ready for next update */

  rewind(tempfp);
  if (fscanf(tempfp,"%f %c",&ftemp,&forc) != 2)
  {
    fprintf(stderr,"Temperature data file not in proper format\n");
    exit(1);
  }
  deg = (int) (ftemp + 0.5);
  t = scale(deg);
  time(&now);
  temp_ptr = (now - temp_time) / 60;

  XDrawPoint(display,win,blackgc,graphx+temp_ptr,t);
  temps[temp_ptr] = deg;
  if (temp_ptr >= graphsx)
  {
    temp_ptr = graphsx - SHIFT_INTERVAL;
    for (i = 0; i <= temp_ptr; i++)
      temps[i] = temps[i+SHIFT_INTERVAL];
    for (; i < graphsx; i++)
      temps[i] = 0;
    temp_mins += SHIFT_INTERVAL;
    if (temp_mins >= (24 * 60))
      temp_mins -= (24 * 60);
    temp_time += (60 * SHIFT_INTERVAL);
    redraw();
  }

  /* put current temp at far right */
  if (o_temp != 0)
    XDrawString(display,win,whitetextgc,winsx-GRAPH_RIG_BORDER+4,
		scale(o_temp)+(FONT_Y/2),s_temp,l_temp);
  sprintf(s_temp,"%d",deg);
  l_temp = strlen(s_temp);
  w_temp = XTextWidth(font_info, s_temp, l_temp);
  XDrawString(display,win,blacktextgc,winsx-GRAPH_RIG_BORDER+4,
	      t+(FONT_Y/2),s_temp,l_temp);

  o_temp = deg;
  XFlush(display);
}

redraw()
{
  XClearArea(display,win,0,0,winsx,winsy,False);
  draw_text(blacktextgc);
  draw_graphics(blackgc);
}

draw_text(gc)
  GC gc;
{
  int i;
  char s_time[10];
  int l_time, w_time;

  /* make time scale */
  for (i = 0; i < graphsx; i++)
  {
    if (((temp_mins+i) % 60) == 0)
    {
      sprintf(s_time,"%d",((temp_mins+i)/60) % 24);
      l_time = strlen(s_time);
      w_time = XTextWidth(font_info, s_time, l_time);
      XDrawString(display,win,gc,graphx+i-(w_time/2),winsy-SCALE_YPOS,
		  s_time,l_time);
    }
  }

  /* make temperature scale */
  for (i = 0; i < graphsy;)
  {
    sprintf(s_time,"%d",(temp_base+i));
    l_time = strlen(s_time);
    w_time = XTextWidth(font_info, s_time, l_time);
    XDrawString(display,win,gc,2,(scale(temp_base+i) + (FONT_Y/2)),
		s_time,l_time);
    i += temp_inc;
  }
  XDrawString(display,win,blacktextgc,winsx-GRAPH_RIG_BORDER+4,
	      scale(o_temp)+(FONT_Y/2),s_temp,l_temp);
}

draw_graphics(gc)
  GC gc;
{
  int i, t;

  for (i = 0; i < graphsx; i++)
  {
    t = temps[i];
    if (t > 0)
      XDrawPoint(display,win,gc,graphx+i,scale(t));
  }
}

int scale(temp)
  int temp;
{
  return(((graphy + graphsy) - ((temp - temp_base) * temp_scale)));
}

TooSmall(gc)
  GC gc;
{
  char *s1 = "Too Small";
  int y_offset, x_offset;

  y_offset = font_info->max_bounds.ascent + 2;
  x_offset = 2;
  XDrawString(display,win,gc,x_offset,y_offset,s1,strlen(s1));
}
